@@ -2,6 +2,7 @@
 ## Copyright (C) 2006 Alberto Pose <apose@alu.itba.edu.ar>
 ## Copyright (C) 2011 Pascal Dupuis <Pascal.Dupuis@worldonline.be>
 ## Copyright (C) 2012 Juan Pablo Carbajal <carbajal@ifi.uzh.ch>
+## Copyright (C) 2016 Pascal Dupuis <cdemills@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -19,6 +20,7 @@
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {@var{s} =} boxplot (@var{data}, @var{notched}, @
 ## @var{symbol}, @var{vertical}, @var{maxwhisker}, @dots{})
+## @deftypefnx {Function File} {@var{s} =} boxplot (@var{data}, @var{group})
 ## @deftypefnx {Function File} {[@dots{} @var{h}]=} boxplot (@dots{})
 ##
 ## Produce a box plot.
@@ -71,7 +73,7 @@
 ## @item 7 @tab Upper confidence limit for median
 ## @end multitable
 ##
-## The returned structure @var{h} has hanldes to the plot elements, allowing
+## The returned structure @var{h} has handles to the plot elements, allowing
 ## customization of the visualization using set/get functions.
 ##
 ## Example
@@ -79,7 +81,7 @@
 ## @example
 ## title ("Grade 3 heights");
 ## axis ([0,3]);
-## tics ("x", 1:2, @{"girls"; "boys"@});
+## set(gca (), "xtick", [1 2], "xticklabel", @{"girls", "boys"@});
 ## boxplot (@{randn(10,1)*5+140, randn(13,1)*8+135@});
 ## @end example
 ##
@@ -98,7 +100,8 @@ function [s hs] = boxplot (data, varargin)
   symbol = ['+', 'o'];
   notched = 0;
   plot_opts = {};
-
+  groups = [];
+  
   %# Optional arguments analysis
   numarg = nargin - 1;
   option_args = ['Notch'; 'Symbol'; 'Vertical'; 'Maxwhisker'];
@@ -106,17 +109,25 @@ function [s hs] = boxplot (data, varargin)
   while (numarg)
     dummy = varargin{indopt++};
     if (!ischar (dummy))
-      %# old way: positional argument
-      switch indopt
-        case 2
-          notched = dummy;
-        case 4
-          vertical = dummy;
-        case 5
-          maxwhisker = dummy;
-        otherwise
-          error("No positional argument allowed at position %d", --indopt);
-      endswitch
+      %# MatLAB allows passing the second argument as a grouping vector
+      if (length (dummy) > 1)
+        if (2 ~= indopt)
+          error ('Boxplot.m: grouping vector may only be passed as second arg');
+        endif
+        groups = dummy;
+      else  
+        %# old way: positional argument
+        switch indopt
+          case 2
+            notched = dummy;
+          case 4
+            vertical = dummy;
+          case 5
+            maxwhisker = dummy;
+          otherwise
+            error("No positional argument allowed at position %d", --indopt);
+        endswitch
+      endif
       numarg--; continue;
     else
       if (3 == indopt && length (dummy) <= 2)
@@ -147,22 +158,34 @@ function [s hs] = boxplot (data, varargin)
   a = 1-notched;
 
   ## figure out how many data sets we have
-  if (iscell (data))
-    nc = length (data);
+  if (isempty (groups))
+    if (iscell (data))
+      nc = length (data);
+    else
+      if (isvector (data)) data = data(:); endif
+      nc = columns (data);
+    endif
+    groups = (1:nc);
   else
-    if (isvector (data)) data = data(:); endif
-    nc = columns (data);
-  endif
+    if (~isvector (data))
+      error ('Boxplot.m: with the formalism (data, group), both must be vectors');
+    end
+    nc = unique (groups); dummy = cell (1, length (nc));
+    for indopt = (1:length (nc))
+      dummy(indopt) = data(groups == nc(indopt));
+    end
+    data = dummy; groups = nc(:).'; nc = length (nc); 
+  end
 
   ## compute statistics
   ## s will contain
   ##    1,5    min and max
   ##    2,3,4  1st, 2nd and 3rd quartile
   ##    6,7    lower and upper confidence intervals for median
-  s = zeros (7,nc);
-  box = zeros (1,nc);
-  whisker_x = ones (2,1)*[1:nc,1:nc];
-  whisker_y = zeros (2,2*nc);
+  s = zeros (7, nc);
+  box = zeros (1, nc);
+  whisker_x = ones (2,1)*[groups, groups];
+  whisker_y = zeros (2, 2*nc);
   outliers_x = [];
   outliers_y = [];
   outliers2_x = [];
@@ -194,15 +217,15 @@ function [s hs] = boxplot (data, varargin)
       ## outliers beyond 1 and 2 inter-quartile ranges
       outliers = col((col < s(2, indi)-IQR & col >= s(2, indi)-2*IQR) | (col > s(4, indi)+IQR & col <= s(4, indi)+2*IQR));
       outliers2 = col(col < s(2, indi)-2*IQR | col > s(4, indi)+2*IQR);
-      outliers_x = [outliers_x; indi*ones(size(outliers))];
+      outliers_x = [outliers_x; groups(indi)*ones(size(outliers))];
       outliers_y = [outliers_y; outliers];
-      outliers2_x = [outliers2_x; indi*ones(size(outliers2))];
+      outliers2_x = [outliers2_x; groups(indi)*ones(size(outliers2))];
       outliers2_y = [outliers2_y; outliers2];
     elseif (1 == nd)
       ## all statistics collapse to the value of the point
       s(:, indi) = col;
       ## single point data sets are plotted as outliers.
-      outliers_x = [outliers_x; indi];
+      outliers_x = [outliers_x; groups(indi)];
       outliers_y = [outliers_y; col];
     else
       ## no statistics if no points
@@ -216,11 +239,11 @@ function [s hs] = boxplot (data, varargin)
   ## Draw a box around the quartiles, with width proportional to the number of
   ## items in the box. Draw notches if desired.
   box *= 0.4/max (box);
-  quartile_x = ones (11,1)*[1:nc] + [-a;-1;-1;1;1;a;1;1;-1;-1;-a]*box;
+  quartile_x = ones (11,1)*groups + [-a;-1;-1;1;1;a;1;1;-1;-1;-a]*box;
   quartile_y = s([3,7,4,4,7,3,6,2,2,6,3],:);
 
   ## Draw a line through the median
-  median_x = ones (2,1)*[1:nc] + [-a;+a]*box;
+  median_x = ones (2,1)*groups + [-a;+a]*box;
   median_y = s([3,3],:);
 
   ## Chop all boxes which don't have enough stats
@@ -302,4 +325,3 @@ endfunction
 %! boxplot ({randn(10,1)*5+140, randn(13,1)*8+135});
 %! set(gca (), "xtick", [1 2], "xticklabel", {"girls", "boys"})
 %! title ("Grade 3 heights");
-
